@@ -1,19 +1,17 @@
-"""Asynchronous request parser. Compatible with Python>=3.5."""
+"""Asynchronous request parser."""
+from __future__ import annotations
+
 import asyncio
 import functools
 import inspect
 import typing
-from collections.abc import Mapping
 
 from marshmallow import Schema, ValidationError
-from marshmallow.fields import Field
 import marshmallow as ma
 
 from webargs import core
 
-Request = typing.TypeVar("Request")
-ArgMap = typing.Union[Schema, typing.Mapping[str, Field]]
-Validate = typing.Union[typing.Callable, typing.Iterable[typing.Callable]]
+AsyncErrorHandler = typing.Callable[..., typing.Awaitable[typing.NoReturn]]
 
 
 class AsyncParser(core.Parser):
@@ -24,20 +22,31 @@ class AsyncParser(core.Parser):
     # TODO: Lots of duplication from core.Parser here. Rethink.
     async def parse(
         self,
-        argmap: ArgMap,
-        req: Request = None,
+        argmap: core.ArgMap,
+        req: core.Request | None = None,
         *,
-        location: str = None,
-        validate: Validate = None,
-        error_status_code: typing.Union[int, None] = None,
-        error_headers: typing.Union[typing.Mapping[str, str], None] = None
-    ) -> typing.Union[typing.Mapping, None]:
+        location: str | None = None,
+        unknown: str | None = core._UNKNOWN_DEFAULT_PARAM,
+        validate: core.ValidateArg = None,
+        error_status_code: int | None = None,
+        error_headers: typing.Mapping[str, str] | None = None,
+    ) -> typing.Mapping | None:
         """Coroutine variant of `webargs.core.Parser`.
 
         Receives the same arguments as `webargs.core.Parser.parse`.
         """
         req = req if req is not None else self.get_default_request()
         location = location or self.location
+        unknown = (
+            unknown
+            if unknown != core._UNKNOWN_DEFAULT_PARAM
+            else (
+                self.unknown
+                if self.unknown != core._UNKNOWN_DEFAULT_PARAM
+                else self.DEFAULT_UNKNOWN_BY_LOCATION.get(location)
+            )
+        )
+        load_kwargs: dict[str, typing.Any] = {"unknown": unknown} if unknown else {}
         if req is None:
             raise ValueError("Must pass req object")
         data = None
@@ -47,11 +56,10 @@ class AsyncParser(core.Parser):
             location_data = await self._load_location_data(
                 schema=schema, req=req, location=location
             )
-            result = schema.load(location_data)
-            data = result.data if core.MARSHMALLOW_VERSION_INFO[0] < 3 else result
+            data = schema.load(location_data, **load_kwargs)
             self._validate_arguments(data, validators)
         except ma.exceptions.ValidationError as error:
-            await self._on_validation_error(
+            await self._async_on_validation_error(
                 error,
                 req,
                 schema,
@@ -80,16 +88,16 @@ class AsyncParser(core.Parser):
             data = {}
         return data
 
-    async def _on_validation_error(
+    async def _async_on_validation_error(
         self,
         error: ValidationError,
-        req: Request,
+        req: core.Request,
         schema: Schema,
         location: str,
         *,
-        error_status_code: typing.Union[int, None],
-        error_headers: typing.Union[typing.Mapping[str, str], None]
-    ) -> None:
+        error_status_code: int | None,
+        error_headers: typing.Mapping[str, str] | None,
+    ) -> typing.NoReturn:
         # rewrite messages to be namespaced under the location which created
         # them
         # e.g. {"json":{"foo":["Not a valid integer."]}}
@@ -97,24 +105,41 @@ class AsyncParser(core.Parser):
         #      {"foo":["Not a valid integer."]}
         error.messages = {location: error.messages}
         error_handler = self.error_callback or self.handle_error
-        await error_handler(
-            error,
-            req,
-            schema,
-            error_status_code=error_status_code,
-            error_headers=error_headers,
-        )
+        # an async error handler was registered, await it
+        if inspect.iscoroutinefunction(error_handler):
+            async_error_handler = typing.cast(AsyncErrorHandler, error_handler)
+            await async_error_handler(
+                error,
+                req,
+                schema,
+                error_status_code=error_status_code,
+                error_headers=error_headers,
+            )
+            # workaround for mypy not understanding `await Awaitable[NoReturn]`
+            # see: https://github.com/python/mypy/issues/8974
+            raise NotImplementedError("unreachable")
+        # the error handler was synchronous (e.g. Parser.handle_error) so it
+        # will raise an error
+        else:
+            error_handler(
+                error,
+                req,
+                schema,
+                error_status_code=error_status_code,
+                error_headers=error_headers,
+            )
 
     def use_args(
         self,
-        argmap: ArgMap,
-        req: typing.Optional[Request] = None,
+        argmap: core.ArgMap,
+        req: core.Request | None = None,
         *,
         location: str = None,
+        unknown=core._UNKNOWN_DEFAULT_PARAM,
         as_kwargs: bool = False,
-        validate: Validate = None,
-        error_status_code: typing.Optional[int] = None,
-        error_headers: typing.Union[typing.Mapping[str, str], None] = None
+        validate: core.ValidateArg = None,
+        error_status_code: int | None = None,
+        error_headers: typing.Mapping[str, str] | None = None,
     ) -> typing.Callable[..., typing.Callable]:
         """Decorator that injects parsed arguments into a view function or method.
 
@@ -124,8 +149,8 @@ class AsyncParser(core.Parser):
         request_obj = req
         # Optimization: If argmap is passed as a dictionary, we only need
         # to generate a Schema once
-        if isinstance(argmap, Mapping):
-            argmap = core.dict2schema(argmap, schema_class=self.schema_class)()
+        if isinstance(argmap, dict):
+            argmap = self.schema_class.from_dict(argmap)()
 
         def decorator(func: typing.Callable) -> typing.Callable:
             req_ = request_obj
@@ -143,6 +168,7 @@ class AsyncParser(core.Parser):
                         argmap,
                         req=req_obj,
                         location=location,
+                        unknown=unknown,
                         validate=validate,
                         error_status_code=error_status_code,
                         error_headers=error_headers,
@@ -165,6 +191,7 @@ class AsyncParser(core.Parser):
                         argmap,
                         req=req_obj,
                         location=location,
+                        unknown=unknown,
                         validate=validate,
                         error_status_code=error_status_code,
                         error_headers=error_headers,
