@@ -1,56 +1,74 @@
+from __future__ import annotations
+
 import functools
-import inspect
 import typing
 import logging
-import warnings
-from collections.abc import Mapping
 import json
 
 import marshmallow as ma
 from marshmallow import ValidationError
 from marshmallow.utils import missing
 
-from webargs.compat import MARSHMALLOW_VERSION_INFO
-from webargs.dict2schema import dict2schema
-from webargs.fields import DelimitedList
+from webargs.multidictproxy import MultiDictProxy
 
 logger = logging.getLogger(__name__)
 
 
 __all__ = [
     "ValidationError",
-    "dict2schema",
-    "is_multiple",
     "Parser",
     "missing",
     "parse_json",
 ]
 
 
-DEFAULT_VALIDATION_STATUS = 422  # type: int
+Request = typing.TypeVar("Request")
+ArgMap = typing.Union[
+    ma.Schema,
+    typing.Dict[str, typing.Union[ma.fields.Field, typing.Type[ma.fields.Field]]],
+    typing.Callable[[Request], ma.Schema],
+]
+ValidateArg = typing.Union[None, typing.Callable, typing.Iterable[typing.Callable]]
+CallableList = typing.List[typing.Callable]
+ErrorHandler = typing.Callable[..., typing.NoReturn]
+# generic type var with no particular meaning
+T = typing.TypeVar("T")
+# type var for callables, to make type-preserving decorators
+C = typing.TypeVar("C", bound=typing.Callable)
+# type var for a callable which is an error handler
+# used to ensure that the error_handler decorator is type preserving
+ErrorHandlerT = typing.TypeVar("ErrorHandlerT", bound=ErrorHandler)
 
 
-def _callable_or_raise(obj):
+# a value used as the default for arguments, so that when `None` is passed, it
+# can be distinguished from the default value
+_UNKNOWN_DEFAULT_PARAM = "_default"
+
+DEFAULT_VALIDATION_STATUS: int = 422
+
+
+def _iscallable(x) -> bool:
+    # workaround for
+    #   https://github.com/python/mypy/issues/9778
+    return callable(x)
+
+
+def _callable_or_raise(obj: T | None) -> T | None:
     """Makes sure an object is callable if it is not ``None``. If not
     callable, a ValueError is raised.
     """
-    if obj and not callable(obj):
-        raise ValueError("{!r} is not callable.".format(obj))
+    if obj and not _iscallable(obj):
+        raise ValueError(f"{obj!r} is not callable.")
     return obj
 
 
-def is_multiple(field):
-    """Return whether or not `field` handles repeated/multi-value arguments."""
-    return isinstance(field, ma.fields.List) and not isinstance(field, DelimitedList)
-
-
-def get_mimetype(content_type):
-    return content_type.split(";")[0].strip() if content_type else None
+def get_mimetype(content_type: str) -> str:
+    return content_type.split(";")[0].strip()
 
 
 # Adapted from werkzeug:
 # https://github.com/mitsuhiko/werkzeug/blob/master/werkzeug/wrappers.py
-def is_json(mimetype):
+def is_json(mimetype: str | None) -> bool:
     """Indicates if this mimetype is JSON or not.  By default a request
     is considered to include JSON data if the mimetype is
     ``application/json`` or ``application/*+json``.
@@ -66,27 +84,29 @@ def is_json(mimetype):
     return False
 
 
-def parse_json(string, *, encoding="utf-8"):
-    if isinstance(string, bytes):
+def parse_json(s: typing.AnyStr, *, encoding: str = "utf-8") -> typing.Any:
+    if isinstance(s, str):
+        decoded = s
+    else:
         try:
-            string = string.decode(encoding)
+            decoded = s.decode(encoding)
         except UnicodeDecodeError as exc:
             raise json.JSONDecodeError(
-                "Bytes decoding error : {}".format(exc.reason),
+                f"Bytes decoding error : {exc.reason}",
                 doc=str(exc.object),
                 pos=exc.start,
-            )
-    return json.loads(string)
+            ) from exc
+    return json.loads(decoded)
 
 
-def _ensure_list_of_callables(obj):
+def _ensure_list_of_callables(obj: typing.Any) -> CallableList:
     if obj:
         if isinstance(obj, (list, tuple)):
-            validators = obj
+            validators = typing.cast(CallableList, list(obj))
         elif callable(obj):
             validators = [obj]
         else:
-            raise ValueError("{!r} is not a callable or list of callables.".format(obj))
+            raise ValueError(f"{obj!r} is not a callable or list of callables.")
     else:
         validators = []
     return validators
@@ -101,20 +121,38 @@ class Parser:
     etc.
 
     :param str location: Default location to use for data
+    :param str unknown: A default value to pass for ``unknown`` when calling the
+        schema's ``load`` method. Defaults to EXCLUDE for non-body
+        locations and RAISE for request bodies. Pass ``None`` to use the
+        schema's setting instead.
     :param callable error_handler: Custom error handler function.
     """
 
     #: Default location to check for data
-    DEFAULT_LOCATION = "json"
+    DEFAULT_LOCATION: str = "json"
+    #: Default value to use for 'unknown' on schema load
+    #  on a per-location basis
+    DEFAULT_UNKNOWN_BY_LOCATION: dict[str, str | None] = {
+        "json": None,
+        "form": None,
+        "json_or_form": None,
+        "querystring": ma.EXCLUDE,
+        "query": ma.EXCLUDE,
+        "headers": ma.EXCLUDE,
+        "cookies": ma.EXCLUDE,
+        "files": ma.EXCLUDE,
+    }
     #: The marshmallow Schema class to use when creating new schemas
-    DEFAULT_SCHEMA_CLASS = ma.Schema
+    DEFAULT_SCHEMA_CLASS: type[ma.Schema] = ma.Schema
     #: Default status code to return for validation errors
-    DEFAULT_VALIDATION_STATUS = DEFAULT_VALIDATION_STATUS
+    DEFAULT_VALIDATION_STATUS: int = DEFAULT_VALIDATION_STATUS
     #: Default error message for validation errors
-    DEFAULT_VALIDATION_MESSAGE = "Invalid value."
+    DEFAULT_VALIDATION_MESSAGE: str = "Invalid value."
+    #: field types which should always be treated as if they set `is_multiple=True`
+    KNOWN_MULTI_FIELDS: list[type] = [ma.fields.List, ma.fields.Tuple]
 
     #: Maps location => method name
-    __location_map__ = {
+    __location_map__: dict[str, str | typing.Callable] = {
         "json": "load_json",
         "querystring": "load_querystring",
         "query": "load_querystring",
@@ -125,34 +163,42 @@ class Parser:
         "json_or_form": "load_json_or_form",
     }
 
-    def __init__(self, location=None, *, error_handler=None, schema_class=None):
+    def __init__(
+        self,
+        location: str | None = None,
+        *,
+        unknown: str | None = _UNKNOWN_DEFAULT_PARAM,
+        error_handler: ErrorHandler | None = None,
+        schema_class: type[ma.Schema] | None = None,
+    ):
         self.location = location or self.DEFAULT_LOCATION
-        self.error_callback = _callable_or_raise(error_handler)
+        self.error_callback: ErrorHandler | None = _callable_or_raise(error_handler)
         self.schema_class = schema_class or self.DEFAULT_SCHEMA_CLASS
+        self.unknown = unknown
 
-    def _get_loader(self, location):
+    def _makeproxy(self, multidict, schema: ma.Schema, cls: type = MultiDictProxy):
+        """Create a multidict proxy object with options from the current parser"""
+        return cls(multidict, schema, known_multi_fields=tuple(self.KNOWN_MULTI_FIELDS))
+
+    def _get_loader(self, location: str) -> typing.Callable:
         """Get the loader function for the given location.
 
         :raises: ValueError if a given location is invalid.
         """
         valid_locations = set(self.__location_map__.keys())
         if location not in valid_locations:
-            msg = "Invalid location argument: {}".format(location)
-            raise ValueError(msg)
+            raise ValueError(f"Invalid location argument: {location}")
 
         # Parsing function to call
         # May be a method name (str) or a function
-        func = self.__location_map__.get(location)
-        if func:
-            if inspect.isfunction(func):
-                function = func
-            else:
-                function = getattr(self, func)
-        else:
-            raise ValueError('Invalid location: "{}"'.format(location))
-        return function
+        func = self.__location_map__[location]
+        if isinstance(func, str):
+            return getattr(self, func)
+        return func
 
-    def _load_location_data(self, *, schema, req, location):
+    def _load_location_data(
+        self, *, schema: ma.Schema, req: Request, location: str
+    ) -> typing.Mapping:
         """Return a dictionary-like object for the location on the given request.
 
         Needs to have the schema in hand in order to correctly handle loading
@@ -168,15 +214,22 @@ class Parser:
         return data
 
     def _on_validation_error(
-        self, error, req, schema, location, *, error_status_code, error_headers
-    ):
+        self,
+        error: ValidationError,
+        req: Request,
+        schema: ma.Schema,
+        location: str,
+        *,
+        error_status_code: int | None,
+        error_headers: typing.Mapping[str, str] | None,
+    ) -> typing.NoReturn:
         # rewrite messages to be namespaced under the location which created
         # them
         # e.g. {"json":{"foo":["Not a valid integer."]}}
         #      instead of
         #      {"foo":["Not a valid integer."]}
         error.messages = {location: error.messages}
-        error_handler = self.error_callback or self.handle_error
+        error_handler: ErrorHandler = self.error_callback or self.handle_error
         error_handler(
             error,
             req,
@@ -185,13 +238,15 @@ class Parser:
             error_headers=error_headers,
         )
 
-    def _validate_arguments(self, data, validators):
+    def _validate_arguments(self, data: typing.Any, validators: CallableList) -> None:
+        # although `data` is typically a Mapping, nothing forbids a `schema.load`
+        # from returning an arbitrary object subject to validators
         for validator in validators:
             if validator(data) is False:
                 msg = self.DEFAULT_VALIDATION_MESSAGE
                 raise ValidationError(msg, data=data)
 
-    def _get_schema(self, argmap, req):
+    def _get_schema(self, argmap: ArgMap, req: Request) -> ma.Schema:
         """Return a `marshmallow.Schema` for the given argmap and request.
 
         :param argmap: Either a `marshmallow.Schema`, `dict`
@@ -206,25 +261,22 @@ class Parser:
             schema = argmap()
         elif callable(argmap):
             schema = argmap(req)
+        elif isinstance(argmap, dict):
+            schema = self.schema_class.from_dict(argmap)()
         else:
-            schema = dict2schema(argmap, schema_class=self.schema_class)()
-        if MARSHMALLOW_VERSION_INFO[0] < 3 and not schema.strict:
-            warnings.warn(
-                "It is highly recommended that you set strict=True on your schema "
-                "so that the parser's error handler will be invoked when expected.",
-                UserWarning,
-            )
+            raise TypeError(f"argmap was of unexpected type {type(argmap)}")
         return schema
 
     def parse(
         self,
-        argmap,
-        req=None,
+        argmap: ArgMap,
+        req: Request | None = None,
         *,
-        location=None,
-        validate=None,
-        error_status_code=None,
-        error_headers=None
+        location: str | None = None,
+        unknown: str | None = _UNKNOWN_DEFAULT_PARAM,
+        validate: ValidateArg = None,
+        error_status_code: int | None = None,
+        error_headers: typing.Mapping[str, str] | None = None,
     ):
         """Main request parsing method.
 
@@ -236,6 +288,10 @@ class Parser:
             Can be any of the values in :py:attr:`~__location_map__`. By
             default, that means one of ``('json', 'query', 'querystring',
             'form', 'headers', 'cookies', 'files', 'json_or_form')``.
+        :param str unknown: A value to pass for ``unknown`` when calling the
+            schema's ``load`` method. Defaults to EXCLUDE for non-body
+            locations and RAISE for request bodies. Pass ``None`` to use the
+            schema's setting instead.
         :param callable validate: Validation function or list of validation functions
             that receives the dictionary of parsed arguments. Validator either returns a
             boolean or raises a :exc:`ValidationError`.
@@ -248,6 +304,17 @@ class Parser:
         """
         req = req if req is not None else self.get_default_request()
         location = location or self.location
+        # precedence order: explicit, instance setting, default per location
+        unknown = (
+            unknown
+            if unknown != _UNKNOWN_DEFAULT_PARAM
+            else (
+                self.unknown
+                if self.unknown != _UNKNOWN_DEFAULT_PARAM
+                else self.DEFAULT_UNKNOWN_BY_LOCATION.get(location)
+            )
+        )
+        load_kwargs: dict[str, typing.Any] = {"unknown": unknown} if unknown else {}
         if req is None:
             raise ValueError("Must pass req object")
         data = None
@@ -257,8 +324,10 @@ class Parser:
             location_data = self._load_location_data(
                 schema=schema, req=req, location=location
             )
-            result = schema.load(location_data)
-            data = result.data if MARSHMALLOW_VERSION_INFO[0] < 3 else result
+            preprocessed_data = self.pre_load(
+                location_data, schema=schema, req=req, location=location
+            )
+            data = schema.load(preprocessed_data, **load_kwargs)
             self._validate_arguments(data, validators)
         except ma.exceptions.ValidationError as error:
             self._on_validation_error(
@@ -269,15 +338,23 @@ class Parser:
                 error_status_code=error_status_code,
                 error_headers=error_headers,
             )
+            raise ValueError(
+                "_on_validation_error hook did not raise an exception"
+            ) from error
         return data
 
-    def get_default_request(self):
+    def get_default_request(self) -> Request | None:
         """Optional override. Provides a hook for frameworks that use thread-local
         request objects.
         """
         return None
 
-    def get_request_from_view_args(self, view, args, kwargs):
+    def get_request_from_view_args(
+        self,
+        view: typing.Callable,
+        args: tuple,
+        kwargs: typing.Mapping[str, typing.Any],
+    ) -> Request | None:
         """Optional override. Returns the request object to be parsed, given a view
         function's args and kwargs.
 
@@ -292,7 +369,12 @@ class Parser:
         return None
 
     @staticmethod
-    def _update_args_kwargs(args, kwargs, parsed_args, as_kwargs):
+    def _update_args_kwargs(
+        args: tuple,
+        kwargs: dict[str, typing.Any],
+        parsed_args: tuple,
+        as_kwargs: bool,
+    ) -> tuple[tuple, typing.Mapping]:
         """Update args or kwargs with parsed_args depending on as_kwargs"""
         if as_kwargs:
             kwargs.update(parsed_args)
@@ -303,15 +385,16 @@ class Parser:
 
     def use_args(
         self,
-        argmap,
-        req=None,
+        argmap: ArgMap,
+        req: Request | None = None,
         *,
-        location=None,
-        as_kwargs=False,
-        validate=None,
-        error_status_code=None,
-        error_headers=None
-    ):
+        location: str | None = None,
+        unknown: str | None = _UNKNOWN_DEFAULT_PARAM,
+        as_kwargs: bool = False,
+        validate: ValidateArg = None,
+        error_status_code: int | None = None,
+        error_headers: typing.Mapping[str, str] | None = None,
+    ) -> typing.Callable[..., typing.Callable]:
         """Decorator that injects parsed arguments into a view function or method.
 
         Example usage with Flask: ::
@@ -325,6 +408,8 @@ class Parser:
             of argname -> `marshmallow.fields.Field` pairs, or a callable
             which accepts a request and returns a `marshmallow.Schema`.
         :param str location: Where on the request to load values.
+        :param str unknown: A value to pass for ``unknown`` when calling the
+            schema's ``load`` method.
         :param bool as_kwargs: Whether to insert arguments as keyword arguments.
         :param callable validate: Validation function that receives the dictionary
             of parsed arguments. If the function returns ``False``, the parser
@@ -338,8 +423,8 @@ class Parser:
         request_obj = req
         # Optimization: If argmap is passed as a dictionary, we only need
         # to generate a Schema once
-        if isinstance(argmap, Mapping):
-            argmap = dict2schema(argmap, schema_class=self.schema_class)()
+        if isinstance(argmap, dict):
+            argmap = self.schema_class.from_dict(argmap)()
 
         def decorator(func):
             req_ = request_obj
@@ -356,6 +441,7 @@ class Parser:
                     argmap,
                     req=req_obj,
                     location=location,
+                    unknown=unknown,
                     validate=validate,
                     error_status_code=error_status_code,
                     error_headers=error_headers,
@@ -388,7 +474,7 @@ class Parser:
         kwargs["as_kwargs"] = True
         return self.use_args(*args, **kwargs)
 
-    def location_loader(self, name):
+    def location_loader(self, name: str) -> typing.Callable[[C], C]:
         """Decorator that registers a function for loading a request location.
         The wrapped function receives a schema and a request.
 
@@ -409,13 +495,13 @@ class Parser:
         :param str name: The name of the location to register.
         """
 
-        def decorator(func):
+        def decorator(func: C) -> C:
             self.__location_map__[name] = func
             return func
 
         return decorator
 
-    def error_handler(self, func):
+    def error_handler(self, func: ErrorHandlerT) -> ErrorHandlerT:
         """Decorator that registers a custom error handling function. The
         function should receive the raised error, request object,
         `marshmallow.Schema` instance used to parse the request, error status code,
@@ -434,7 +520,7 @@ class Parser:
 
 
             @parser.error_handler
-            def handle_error(error, req, schema, *, status_code, headers):
+            def handle_error(error, req, schema, *, error_status_code, error_headers):
                 raise CustomError(error.messages)
 
         :param callable func: The error callback to register.
@@ -442,7 +528,27 @@ class Parser:
         self.error_callback = func
         return func
 
-    def _handle_invalid_json_error(self, error, req, *args, **kwargs):
+    def pre_load(
+        self,
+        location_data: typing.Mapping,
+        *,
+        schema: ma.Schema,
+        req: Request,
+        location: str,
+    ) -> typing.Mapping:
+        """A method of the parser which can transform data after location
+        loading is done. By default it does nothing, but users can subclass
+        parsers and override this method.
+        """
+        return location_data
+
+    def _handle_invalid_json_error(
+        self,
+        error: json.JSONDecodeError | UnicodeDecodeError,
+        req: Request,
+        *args,
+        **kwargs,
+    ) -> typing.NoReturn:
         """Internal hook for overriding treatment of JSONDecodeErrors.
 
         Invoked by default `load_json` implementation.
@@ -452,7 +558,7 @@ class Parser:
         """
         raise error
 
-    def load_json(self, req, schema):
+    def load_json(self, req: Request, schema: ma.Schema) -> typing.Any:
         """Load JSON from a request object or return `missing` if no value can
         be found.
         """
@@ -470,7 +576,7 @@ class Parser:
         except UnicodeDecodeError as exc:
             return self._handle_invalid_json_error(exc, req)
 
-    def load_json_or_form(self, req, schema):
+    def load_json_or_form(self, req: Request, schema: ma.Schema):
         """Load data from a request, accepting either JSON or form-encoded
         data.
 
@@ -484,7 +590,7 @@ class Parser:
 
     # Abstract Methods
 
-    def _raw_load_json(self, req):
+    def _raw_load_json(self, req: Request):
         """Internal hook method for implementing load_json()
 
         Get a request body for feeding in to `load_json`, and parse it either
@@ -500,36 +606,43 @@ class Parser:
         """
         return missing
 
-    def load_querystring(self, req, schema):
+    def load_querystring(self, req: Request, schema: ma.Schema):
         """Load the query string of a request object or return `missing` if no
         value can be found.
         """
         return missing
 
-    def load_form(self, req, schema):
+    def load_form(self, req: Request, schema: ma.Schema):
         """Load the form data of a request object or return `missing` if no
         value can be found.
         """
         return missing
 
-    def load_headers(self, req, schema):
-        """Load the headers or return `missing` if no value can be found.
-        """
+    def load_headers(self, req: Request, schema: ma.Schema):
+        """Load the headers or return `missing` if no value can be found."""
         return missing
 
-    def load_cookies(self, req, schema):
+    def load_cookies(self, req: Request, schema: ma.Schema):
         """Load the cookies from the request or return `missing` if no value
         can be found.
         """
         return missing
 
-    def load_files(self, req, schema):
+    def load_files(self, req: Request, schema: ma.Schema):
         """Load files from the request or return `missing` if no values can be
         found.
         """
         return missing
 
-    def handle_error(self, error, req, schema, *, error_status_code, error_headers):
+    def handle_error(
+        self,
+        error: ValidationError,
+        req: Request,
+        schema: ma.Schema,
+        *,
+        error_status_code: int,
+        error_headers: typing.Mapping[str, str],
+    ) -> typing.NoReturn:
         """Called if an error occurs while parsing args. By default, just logs and
         raises ``error``.
         """
