@@ -9,21 +9,25 @@ Example: ::
 
     app = Flask(__name__)
 
-    hello_args = {
-        'name': fields.Str(required=True)
+    user_detail_args = {
+        'per_page': fields.Int()
     }
 
-    @app.route('/')
-    @use_args(hello_args)
-    def index(args):
-        return 'Hello ' + args['name']
+    @app.route("/user/<int:uid>")
+    @use_args(user_detail_args)
+    def user_detail(args, uid):
+        return ("The user page for user {uid}, showing {per_page} posts.").format(
+            uid=uid, per_page=args["per_page"]
+        )
 """
+from __future__ import annotations
+
 import flask
 from werkzeug.exceptions import HTTPException
 
+import marshmallow as ma
+
 from webargs import core
-from webargs.compat import MARSHMALLOW_VERSION_INFO
-from webargs.multidictproxy import MultiDictProxy
 
 
 def abort(http_status_code, exc=None, **kwargs):
@@ -47,6 +51,11 @@ def is_json_request(req):
 class FlaskParser(core.Parser):
     """Flask request argument parser."""
 
+    DEFAULT_UNKNOWN_BY_LOCATION: dict[str, str | None] = {
+        "view_args": ma.RAISE,
+        "path": ma.RAISE,
+        **core.Parser.DEFAULT_UNKNOWN_BY_LOCATION,
+    }
     __location_map__ = dict(
         view_args="load_view_args",
         path="load_view_args",
@@ -72,15 +81,15 @@ class FlaskParser(core.Parser):
 
     def load_querystring(self, req, schema):
         """Return query params from the request as a MultiDictProxy."""
-        return MultiDictProxy(req.args, schema)
+        return self._makeproxy(req.args, schema)
 
     def load_form(self, req, schema):
         """Return form values from the request as a MultiDictProxy."""
-        return MultiDictProxy(req.form, schema)
+        return self._makeproxy(req.form, schema)
 
     def load_headers(self, req, schema):
         """Return headers from the request as a MultiDictProxy."""
-        return MultiDictProxy(req.headers, schema)
+        return self._makeproxy(req.headers, schema)
 
     def load_cookies(self, req, schema):
         """Return cookies from the request."""
@@ -88,23 +97,13 @@ class FlaskParser(core.Parser):
 
     def load_files(self, req, schema):
         """Return files from the request as a MultiDictProxy."""
-        return MultiDictProxy(req.files, schema)
+        return self._makeproxy(req.files, schema)
 
     def handle_error(self, error, req, schema, *, error_status_code, error_headers):
         """Handles errors during parsing. Aborts the current HTTP request and
         responds with a 422 error.
         """
         status_code = error_status_code or self.DEFAULT_VALIDATION_STATUS
-        # on marshmallow 2, a many schema receiving a non-list value will
-        # produce this specific error back -- reformat it to match the
-        # marshmallow 3 message so that Flask can properly encode it
-        messages = error.messages
-        if (
-            MARSHMALLOW_VERSION_INFO[0] < 3
-            and schema.many
-            and messages == {0: {}, "_schema": ["Invalid input type."]}
-        ):
-            messages.pop(0)
         abort(
             status_code,
             exc=error,
@@ -114,7 +113,7 @@ class FlaskParser(core.Parser):
         )
 
     def get_default_request(self):
-        """Override to use Flask's thread-local request objec by default"""
+        """Override to use Flask's thread-local request object by default"""
         return flask.request
 
 
